from functools import wraps

from google_sheets_telegram_utils.exceptions import RowDoesNotExistException
from google_sheets_telegram_utils.utils.classes import TelegramUser
from google_sheets_telegram_utils.utils.google_sheet_utils import GoogleSheetConnector


def active_user(google_connector: GoogleSheetConnector):
    def decorator(func):
        @wraps(func)
        def wrapper(update, context):
            pk = update.message.from_user.id
            try:
                data = google_connector.get_row_by_id(pk=pk)
            except RowDoesNotExistException:
                context.bot.send_message(
                    chat_id=update.effective_chat.id,
                    text="Sorry, you are not registered yet",
                )
            else:
                user = TelegramUser(data)
                if not user.is_activated:
                    context.bot.send_message(
                        chat_id=update.effective_chat.id,
                        text="Sorry, Admin has not activated you yet",
                    )
                else:
                    return func(update, context)
        return wrapper
    return decorator
